@@ -11,11 +11,12 @@
 # under the License.
 
 import copy
-import json
 import mock
 import six
 from webob import exc
 
+from oslo_serialization import jsonutils
+
 from senlin.api.middleware import fault
 from senlin.api.openstack.v1 import policies
 from senlin.common import exception as senlin_exc
@@ -26,38 +27,6 @@ from senlin.tests.unit.apiv1 import shared
 from senlin.tests.unit.common import base
 
 
-class PolicyDataTest(base.SenlinTestCase):
-    def test_policy_data(self):
-        body = {
-            'name': 'test_policy',
-            'spec': {
-                'type': 'policy_type',
-                'version': '1.0',
-                'properties': {
-                    'param1': 'value1',
-                    'param2': 'value2',
-                }
-            },
-            'level': 10,
-            'cooldown': 60,
-        }
-
-        data = policies.PolicyData(body)
-        self.assertEqual('test_policy', data.name())
-        self.assertEqual(body['spec'], data.spec())
-        self.assertEqual(10, data.level())
-        self.assertEqual(60, data.cooldown())
-
-    def test_required_fields_missing(self):
-        body = {'not a policy name': 'wibble'}
-
-        data = policies.PolicyData(body)
-        self.assertRaises(exc.HTTPBadRequest, data.name)
-        self.assertRaises(exc.HTTPBadRequest, data.spec)
-        self.assertIsNone(data.level())
-        self.assertIsNone(data.cooldown())
-
-
 @mock.patch.object(policy, 'enforce')
 class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
     def setUp(self):
@@ -69,10 +38,6 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
         cfgopts = DummyConfig()
         self.controller = policies.PolicyController(options=cfgopts)
 
-    def test_policy_default(self, mock_enforce):
-        req = self._get('/policies')
-        self.assertRaises(exc.HTTPNotFound, self.controller.default, req)
-
     def test_policy_index_normal(self, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'index', True)
         req = self._get('/policies')
@@ -86,22 +51,18 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
                     u'param_1': u'value1',
                     u'param_2': u'value2',
                 },
-                u'level': 30,
                 u'created_time': u'2015-02-24T19:17:22Z',
                 u'updated_time': None,
-                u'deleted_time': None,
-                u'cooldown': 60,
             }
         ]
 
         mock_call = self.patchobject(rpc_client.EngineClient, 'call',
                                      return_value=engine_resp)
 
-        result = self.controller.index(req, tenant_id=self.project)
+        result = self.controller.index(req)
 
-        default_args = {'limit': None, 'marker': None,
-                        'sort_keys': None, 'sort_dir': None,
-                        'filters': None, 'show_deleted': False}
+        default_args = {'limit': None, 'marker': None, 'sort': None,
+                        'filters': None, 'project_safe': True}
 
         mock_call.assert_called_with(req.context,
                                      ('policy_list', default_args))
@@ -112,46 +73,57 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
     def test_policy_index_whitelists_params(self, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'index', True)
         params = {
+            'name': 'FAKE',
+            'type': 'TYPE',
             'limit': 20,
             'marker': 'fake marker',
-            'sort_keys': 'fake sort keys',
-            'sort_dir': 'fake sort dir',
-            'show_deleted': False,
-            'balrog': 'you shall not pass!'
+            'sort': 'fake sorting string',
+            'global_project': True,
         }
         req = self._get('/policies', params=params)
 
         mock_call = self.patchobject(rpc_client.EngineClient, 'call')
         mock_call.return_value = []
 
-        self.controller.index(req, tenant_id=self.project)
+        self.controller.index(req)
 
         rpc_call_args, w = mock_call.call_args
         engine_args = rpc_call_args[1][1]
 
-        self.assertEqual(6, len(engine_args))
+        self.assertEqual(5, len(engine_args))
         self.assertIn('limit', engine_args)
         self.assertIn('marker', engine_args)
-        self.assertIn('sort_keys', engine_args)
-        self.assertIn('sort_dir', engine_args)
+        self.assertIn('sort', engine_args)
         self.assertIn('filters', engine_args)
-        self.assertIn('show_deleted', engine_args)
-        self.assertNotIn('tenant_safe', engine_args)
-        self.assertNotIn('balrog', engine_args)
+        self.assertIn('project_safe', engine_args)
+
+    def test_policy_index_whitelist_bad_params(self, mock_enforce):
+        self._mock_enforce_setup(mock_enforce, 'index', True)
+        params = {
+            'balrog': 'fake_value'
+        }
+        req = self._get('/policies', params=params)
+
+        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
+        mock_call.return_value = []
+
+        ex = self.assertRaises(exc.HTTPBadRequest,
+                               self.controller.index, req)
+        self.assertEqual("Invalid parameter balrog", six.text_type(ex))
+        self.assertFalse(mock_call.called)
 
     def test_policy_index_whitelist_filter_params(self, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'index', True)
         params = {
             'type': 'some_type',
             'name': 'fake name',
-            'balrog': 'you shall not pass!'
         }
         req = self._get('/policies', params=params)
 
         mock_call = self.patchobject(rpc_client.EngineClient, 'call')
         mock_call.return_value = []
 
-        self.controller.index(req, tenant_id=self.project)
+        self.controller.index(req)
 
         rpc_call_args, w = mock_call.call_args
         engine_args = rpc_call_args[1][1]
@@ -161,41 +133,19 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
         self.assertEqual(2, len(filters))
         self.assertIn('name', filters)
         self.assertIn('type', filters)
-        self.assertNotIn('balrog', filters)
 
-    def test_policy_index_show_deleted_false(self, mock_enforce):
-        mock_call = self.patchobject(rpc_client.EngineClient, 'policy_list',
-                                     return_value=[])
-
-        params = {'show_deleted': 'False'}
+    def test_policy_index_whitelist_filter_bad_params(self, mock_enforce):
+        self._mock_enforce_setup(mock_enforce, 'index', True)
+        params = {
+            'balrog': 'fake_value'
+        }
         req = self._get('/policies', params=params)
-        self.controller.index(req, tenant_id=self.project)
-        mock_call.assert_called_once_with(mock.ANY,
-                                          filters=mock.ANY,
-                                          show_deleted=False)
 
-    def test_policy_index_show_deleted_true(self, mock_enforce):
-        mock_call = self.patchobject(rpc_client.EngineClient, 'policy_list',
-                                     return_value=[])
+        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
 
-        params = {'show_deleted': 'True'}
-        req = self._get('/policies', params=params)
-        self.controller.index(req, tenant_id=self.project)
-        mock_call.assert_called_once_with(mock.ANY,
-                                          filters=mock.ANY,
-                                          show_deleted=True)
-
-    def test_policy_index_show_deleted_non_bool(self, mock_enforce):
-        mock_call = self.patchobject(rpc_client.EngineClient, 'policy_list',
-                                     return_value=[])
-
-        params = {'show_deleted': 'yes'}
-        req = self._get('/policies', params=params)
-        ex = self.assertRaises(senlin_exc.InvalidParameter,
-                               self.controller.index, req,
-                               tenant_id=self.project)
-        self.assertIn("Invalid value 'yes' specified for 'show_deleted'",
-                      six.text_type(ex))
+        ex = self.assertRaises(exc.HTTPBadRequest,
+                               self.controller.index, req)
+        self.assertEqual("Invalid parameter balrog", six.text_type(ex))
         self.assertFalse(mock_call.called)
 
     def test_policy_index_limit_non_int(self, mock_enforce):
@@ -205,8 +155,7 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
         params = {'limit': 'abc'}
         req = self._get('/policies', params=params)
         ex = self.assertRaises(senlin_exc.InvalidParameter,
-                               self.controller.index, req,
-                               tenant_id=self.project)
+                               self.controller.index, req)
         self.assertIn("Invalid value 'abc' specified for 'limit'",
                       six.text_type(ex))
         self.assertFalse(mock_call.called)
@@ -217,7 +166,7 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
 
         resp = shared.request_with_middleware(fault.FaultWrapper,
                                               self.controller.index,
-                                              req, tenant_id=self.project)
+                                              req)
         self.assertEqual(403, resp.status_int)
         self.assertIn('403 Forbidden', six.text_type(resp))
 
@@ -234,8 +183,6 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
                         'param_2': 2,
                     }
                 },
-                'level': 30,
-                'cooldown': 60,
             }
         }
 
@@ -251,15 +198,13 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
                     'param_2': 2,
                 },
             },
-            'level': 30,
-            'cooldown': 60,
         }
 
-        req = self._post('/policies', json.dumps(body))
+        req = self._post('/policies', jsonutils.dumps(body))
         mock_call = self.patchobject(rpc_client.EngineClient, 'call',
                                      return_value=engine_response)
 
-        resp = self.controller.create(req, tenant_id=self.project, body=body)
+        resp = self.controller.create(req, body=body)
 
         mock_call.assert_called_with(
             req.context,
@@ -270,8 +215,6 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
                     'version': '1.0',
                     'properties': {'param_1': 'value1', 'param_2': 2}
                 },
-                'level': 30,
-                'cooldown': 60
             })
         )
 
@@ -282,12 +225,11 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
         self._mock_enforce_setup(mock_enforce, 'create', True)
         body = {'name': 'test_policy'}
 
-        req = self._post('/policies', json.dumps(body))
+        req = self._post('/policies', jsonutils.dumps(body))
         mock_call = self.patchobject(rpc_client.EngineClient, 'call')
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.create,
-                               req, tenant_id=self.project,
-                               body=body)
+                               req, body=body)
 
         self.assertEqual("Malformed request data, missing 'policy' key "
                          "in request body.", six.text_type(ex))
@@ -306,7 +248,7 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
                 },
             }
         }
-        req = self._post('/policies', json.dumps(body))
+        req = self._post('/policies', jsonutils.dumps(body))
 
         msg = 'Spec validation error (param): value'
         error = senlin_exc.SpecValidationFailed(message=msg)
@@ -315,17 +257,13 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
 
         resp = shared.request_with_middleware(fault.FaultWrapper,
                                               self.controller.create,
-                                              req, tenant_id=self.project,
-                                              body=body)
+                                              req, body=body)
 
         expected_args = body['policy']
-        expected_args['cooldown'] = None
-        expected_args['level'] = None
         mock_call.assert_called_once_with(req.context,
                                           ('policy_create', expected_args))
         self.assertEqual(400, resp.json['code'])
         self.assertEqual('SpecValidationFailed', resp.json['error']['type'])
-        self.assertIsNone(resp.json['error']['traceback'])
 
     def test_policy_create_denied_policy(self, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'create', False)
@@ -340,10 +278,9 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
             }
         }
 
-        req = self._post('/policies', json.dumps(body))
+        req = self._post('/policies', jsonutils.dumps(body))
         resp = shared.request_with_middleware(fault.FaultWrapper,
-                                              self.controller.create,
-                                              req, tenant_id=self.project)
+                                              self.controller.create, req)
         self.assertEqual(403, resp.status_int)
         self.assertIn('403 Forbidden', six.text_type(resp))
 
@@ -364,18 +301,14 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
                     u'param_2': u'value2',
                 }
             },
-            u'level': 30,
             u'created_time': u'2015-02-24T19:17:22Z',
             u'updated_time': None,
-            u'deleted_time': None,
-            u'cooldown': 60,
         }
 
         mock_call = self.patchobject(rpc_client.EngineClient, 'call',
                                      return_value=engine_resp)
 
-        result = self.controller.get(req, tenant_id=self.project,
-                                     policy_id=pid)
+        result = self.controller.get(req, policy_id=pid)
 
         mock_call.assert_called_with(req.context,
                                      ('policy_get', {'identity': pid}))
@@ -394,8 +327,7 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
 
         resp = shared.request_with_middleware(fault.FaultWrapper,
                                               self.controller.get,
-                                              req, tenant_id=self.project,
-                                              policy_id=pid)
+                                              req, policy_id=pid)
 
         self.assertEqual(404, resp.json['code'])
         self.assertEqual('PolicyNotFound', resp.json['error']['type'])
@@ -407,8 +339,7 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
 
         resp = shared.request_with_middleware(fault.FaultWrapper,
                                               self.controller.get,
-                                              req, tenant_id=self.project,
-                                              policy_id=pid)
+                                              req, policy_id=pid)
 
         self.assertEqual(403, resp.status_int)
         self.assertIn('403 Forbidden', six.text_type(resp))
@@ -419,13 +350,11 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
         body = {
             'policy': {
                 'name': 'policy-2',
-                'level': 20,
-                'cooldown': 70,
             }
         }
 
         req = self._put('/policies/%(policy_id)s' % {'policy_id': pid},
-                        json.dumps(body))
+                        jsonutils.dumps(body))
 
         engine_resp = {
             u'id': pid,
@@ -435,63 +364,51 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
                 u'param_1': u'value1',
                 u'param_2': u'value3',
             },
-            u'level': 60,
             u'created_time': u'2015-02-25T16:20:13Z',
             u'updated_time': None,
-            u'deleted_time': None,
-            u'cooldown': 70,
         }
 
         mock_call = self.patchobject(rpc_client.EngineClient, 'call',
                                      return_value=engine_resp)
-        result = self.controller.update(req, tenant_id=self.project,
-                                        policy_id=pid,
-                                        body=body)
+        result = self.controller.update(req, policy_id=pid, body=body)
 
         args = copy.deepcopy(body['policy'])
         args['identity'] = pid
-        args['level'] = 20
-        args['cooldown'] = 70
         mock_call.assert_called_with(req.context, ('policy_update', args))
 
         expected = {'policy': engine_resp}
         self.assertEqual(expected, result)
 
-    def test_policy_update_with_no_level(self, mock_enforce):
+    def test_policy_update_with_no_name(self, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'update', True)
         pid = 'aaaa-bbbb-cccc'
-        body = {'policy': {'cooldown': 70}}
+        body = {'policy': {}}
 
-        req = self._put('/policies/%(pid)s' % {'pid': pid}, json.dumps(body))
+        req = self._put('/policies/%(pid)s' % {'pid': pid},
+                        jsonutils.dumps(body))
 
         engine_resp = mock.Mock()
         mock_call = self.patchobject(rpc_client.EngineClient, 'call',
                                      return_value=engine_resp)
-        result = self.controller.update(req, tenant_id=self.project,
-                                        policy_id=pid, body=body)
+        ex = self.assertRaises(exc.HTTPBadRequest,
+                               self.controller.update,
+                               req, policy_id=pid, body=body)
 
-        args = {
-            'identity': pid,
-            'name': None,
-            'level': None,
-            'cooldown': 70,
-        }
-        mock_call.assert_called_with(req.context, ('policy_update', args))
-
-        expected = {'policy': engine_resp}
-        self.assertEqual(expected, result)
+        self.assertEqual("Policy name not specified.",
+                         six.text_type(ex))
+        self.assertFalse(mock_call.called)
 
     def test_policy_update_with_bad_body(self, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'update', True)
         pid = 'aaaa-bbbb-cccc'
         body = {'foo': 'bar'}
-        req = self._patch('/policies/%(pid)s' % {'pid': pid}, json.dumps(body))
+        req = self._patch('/policies/%(pid)s' % {'pid': pid},
+                          jsonutils.dumps(body))
 
         mock_call = self.patchobject(rpc_client.EngineClient, 'call')
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.update,
-                               req, tenant_id=self.project, policy_id=pid,
-                               body=body)
+                               req, policy_id=pid, body=body)
 
         self.assertEqual("Malformed request data, missing 'policy' key in "
                          "request body.", six.text_type(ex))
@@ -501,17 +418,16 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
         self._mock_enforce_setup(mock_enforce, 'update', True)
         pid = 'aaaa-bbbb-cccc'
         body = {
-            'policy': {'spec': {'param_2': 'value3'}, 'level': 50}
+            'policy': {'spec': {'param_2': 'value3'}}
         }
 
         req = self._patch('/policies/%(policy_id)s' % {'policy_id': pid},
-                          json.dumps(body))
+                          jsonutils.dumps(body))
 
         mock_call = self.patchobject(rpc_client.EngineClient, 'call')
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.update,
-                               req, tenant_id=self.project, policy_id=pid,
-                               body=body)
+                               req, policy_id=pid, body=body)
 
         msg = _("Updating the spec of a policy is not supported because "
                 "it may cause state conflicts in engine.")
@@ -525,11 +441,10 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
         body = {
             'policy': {
                 'name': 'new_policy',
-                'level': 70,
             }
         }
         req = self._patch('/policies/%(policy_id)s' % {'policy_id': pid},
-                          json.dumps(body))
+                          jsonutils.dumps(body))
 
         error = senlin_exc.PolicyNotFound(policy=pid)
         mock_call = self.patchobject(rpc_client.EngineClient, 'call')
@@ -537,9 +452,7 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
 
         resp = shared.request_with_middleware(fault.FaultWrapper,
                                               self.controller.update,
-                                              req, tenant_id=self.project,
-                                              policy_id=pid,
-                                              body=body)
+                                              req, policy_id=pid, body=body)
 
         self.assertEqual(404, resp.json['code'])
         self.assertEqual('PolicyNotFound', resp.json['error']['type'])
@@ -551,13 +464,11 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
             'policy': {'name': 'test_policy', 'spec': {'param5': 'value5'}},
         }
         req = self._put('/policies/%(policy_id)s' % {'policy_id': pid},
-                        json.dumps(body))
+                        jsonutils.dumps(body))
 
         resp = shared.request_with_middleware(fault.FaultWrapper,
                                               self.controller.update,
-                                              req, tenant_id=self.project,
-                                              policy_id=pid,
-                                              body=body)
+                                              req, policy_id=pid, body=body)
 
         self.assertEqual(403, resp.status_int)
         self.assertIn('403 Forbidden', six.text_type(resp))
@@ -570,8 +481,7 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
         mock_call = self.patchobject(rpc_client.EngineClient, 'call',
                                      return_value=None)
 
-        self.assertRaises(exc.HTTPNoContent, self.controller.delete,
-                          req, tenant_id=self.project, policy_id=pid)
+        self.controller.delete(req, policy_id=pid)
 
         mock_call.assert_called_with(
             req.context, ('policy_delete', {'identity': pid}))
@@ -587,8 +497,7 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
 
         resp = shared.request_with_middleware(fault.FaultWrapper,
                                               self.controller.delete,
-                                              req, tenant_id=self.project,
-                                              policy_id=pid)
+                                              req, policy_id=pid)
 
         self.assertEqual(404, resp.json['code'])
         self.assertEqual('PolicyNotFound', resp.json['error']['type'])
@@ -600,8 +509,7 @@ class PolicyControllerTest(shared.ControllerTest, base.SenlinTestCase):
 
         resp = shared.request_with_middleware(fault.FaultWrapper,
                                               self.controller.delete,
-                                              req, tenant_id=self.project,
-                                              policy_id=pid)
+                                              req, policy_id=pid)
 
         self.assertEqual(403, resp.status_int)
         self.assertIn('403 Forbidden', six.text_type(resp))
